==> toycars-0.2.5.tar.gz: toycars-0.2.5/src/Player.h <==
/*
	Copyright (C) 2005 by Ruben Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  Created by Ruben Zilibowitz on 2/01/05.
 */

#ifndef PLAYER_H
#define PLAYER_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "ToyCar.h"

// PlayerEventType - describes the types of events a player can receive
typedef unsigned char PlayerEventType;

// Constants for different event types
const PlayerEventType kPlayerKeyUp = 0;
const PlayerEventType kPlayerKeyDown = 1;
const PlayerEventType kPlayerPaddleMove = 2;

// PlayerEvent - base class for player events
struct PlayerEvent
{
	PlayerEventType type;
};

// PlayerControl - describes the controls a player has
typedef unsigned char PlayerControl;

// PlayerEventPaddle - an event of a "paddle" changing value. Eg mouse moved or joystick moved.
struct PlayerEventPaddle : public PlayerEvent
{
	PlayerControl control;
	unsigned short value;
};

// PlayerKey - describes the keys a player has
typedef unsigned char PlayerKey;

// PlayerEventKey - an event of a key press or release
struct PlayerEventKey : public PlayerEvent
{
	PlayerKey key;
};

// the controls
enum
{
	kSteerWheel = 0,
	kThrottle,
	kBrake
};

// the keys
enum
{
	kSteerWheelLeft = 0,
	kSteerWheelRight,
	kThrottleMore,
	kThrottleLess,
	kBrakeMore,
	kBrakeLess,
	kNumKeys
};

// Player class

class Player
{
public:
			Player(ToyCar& inCar);
	
	void	doEvent(PlayerEvent* event);
	void	integrateKeys(double dt);
	
	ToyCar&	car;
	
	char	updateLaps(const Tuple& lineA, const Tuple& lineB, double curTime);
	short	getLaps()	{ return laps; }
	double	getBestLapTime()	{ return bestLapTime; }
	double	getCourseTime()		{ return courseTime; }
	
protected:
	void	keyUp(PlayerKey key) { keys[key] = false; }
	void	keyDown(PlayerKey key) { keys[key] = true; }
	void	paddleMove(PlayerControl which, unsigned short value);
	
	// key states
	bool	keys[kNumKeys];
	
	// lap counting variables
	char	lapFlag;
	short	laps;
	double	curLap, prevLap;
	
	// times
	double	bestLapTime;
	double	courseTime;
};

#endif

==> toycars-0.3.0.tar.gz: toycars-0.3.0/src/Player.h <==
/*
	Copyright (C) 2005 by Ruben Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  Created by Ruben Zilibowitz on 2/01/05.
 */

#ifndef PLAYER_H
#define PLAYER_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "ToyCar.h"
#include "GameConstants.h"

// PlayerEventType - describes the types of events a player can receive
typedef unsigned char PlayerEventType;

// Constants for different event types
const PlayerEventType kPlayerKeyUp = 0;
const PlayerEventType kPlayerKeyDown = 1;
const PlayerEventType kPlayerPaddleMove = 2;

// PlayerEvent - base class for player events
struct PlayerEvent
{
	PlayerEventType type;
};

// PlayerControl - describes the controls a player has
typedef unsigned char PlayerControl;

// PlayerEventPaddle - an event of a "paddle" changing value. Eg mouse moved or joystick moved.
struct PlayerEventPaddle : public PlayerEvent
{
	PlayerControl control;
	unsigned short value;
};

// PlayerKey - describes the keys a player has
typedef unsigned char PlayerKey;

// PlayerEventKey - an event of a key press or release
struct PlayerEventKey : public PlayerEvent
{
	PlayerKey key;
};

// the controls
enum
{
	kSteerWheel = 0,
	kThrottle,
	kBrake
};

// the keys
enum
{
	kSteerWheelLeft = 0,
	kSteerWheelRight,
	kThrottleMore,
	kThrottleLess,
	kBrakeMore,
	kBrakeLess,
	kNumKeys
};

// constants

const double kMaxSteerAngle = kPi / 4.0;			// rad

// Player class

class Player
{
public:
			Player(ToyCar& inCar, list<Tuple> *path, int inPlayerNumber);
	virtual ~Player() {}
	
	virtual void	doEvent(PlayerEvent* event);
	virtual void	integrateKeys(double dt);
	
	ToyCar&	car;
	
	char	updateLaps(const Tuple& lineA, const Tuple& lineB, double curTime);
	short	getLaps()	{ return laps; }
	double	getBestLapTime()	{ return bestLapTime; }
	double	getCourseTime()		{ return courseTime; }
	double  getCurLap() { return curLap; }
    short	getRevolutions() { return revolutions; }
    void    reset(const Tuple& lineA, const Tuple& lineB, const Player *winner);
	void	updatePathLocation();
	void	computePathLocation();
    
	double getPathDistance() { return (pathIndex + pathSegmentDistance)/pathSize; }
	Tuple getPathLocation() { return pathClosestPoint; }
	Tuple getPathVector();
	int getPlayerNumber() { return playerNumber; }
	
protected:
	void	keyUp(PlayerKey key) { keys[key] = false; }
	void	keyDown(PlayerKey key) { keys[key] = true; }
	void	paddleMove(PlayerControl which, unsigned short value);
	bool	testPathPoint(list<Tuple>::const_iterator Vb);
	bool	testPathSegment(list<Tuple>::const_iterator Va, list<Tuple>::const_iterator Vb);
	
	// key states
	bool	keys[kNumKeys];
	
	// lap counting variables
	char	lapFlag;
	short	laps;
	double	curLap, prevLap;
	short   revolutions;
    
	// times
	double	bestLapTime;
	double	courseTime;
	
	// path information
	const list<Tuple>::const_iterator pathEnd;
	list<Tuple>::const_iterator pathA, pathB;
	int pathIndex;
	double pathSegmentDistance;
	double pathClosestPerp;
	Tuple pathClosestPoint;
	const int pathSize;
	
	// id number
	const int playerNumber;
};

#endif
